/***
 * Excerpted from "Programming Sound with Pure Data",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/thsound for more book information.
***/
/*
 *  Copyright (c) 2012 Peter Brinkmann (peter.brinkmann@gmail.com)
 *
 *  For information on usage and redistribution, and for a DISCLAIMER OF ALL
 *  WARRANTIES, see the file, "LICENSE.txt," in this distribution.
 */

#ifndef __OPENSL_RING_BUFFER_H__
#define __OPENSL_RING_BUFFER_H__

// Simple lock-free ring buffer implementation for one writer thread and one
// consumer thread.
typedef struct ring_buffer {
    int size;
    char *buf_ptr;
    int write_idx;
    int read_idx;
} ring_buffer;

// Creates a ring buffer (returns NULL on failure).
ring_buffer *rb_create(int size);

// Deletes a ring buffer.
void rb_free(ring_buffer *buffer);

// Returns the number of bytes that can currently be written; safe to be called
// from any thread.
int rb_available_to_write(ring_buffer *buffer);

// Returns the number of bytes that can currently be read; safe to be called
// from any thread.
int rb_available_to_read(ring_buffer *buffer);

// Writes the given number of bytes from src to the ring buffer if the ring
// buffer has enough space. Only to be called from a single writer thread.
// Returns 0 on success.
int rb_write_to_buffer(ring_buffer *buffer, const char *src, int len);

// Reads the given number of bytes fromthe ring buffer to dest if the ring
// buffer has enough data. Only to be called from a single reader thread.
// Returns 0 on success.
int rb_read_from_buffer(ring_buffer *buffer, char *dest, int len);

#endif
